//
//  SBMediaOverLayer.m
//  SBTextStoryBorad
//
//  Created by qyb on 2017/7/11.
//  Copyright © 2017年 qyb. All rights reserved.
//

#import "SBMediaOverLayer.h"
#import <SDWebImage/UIImageView+WebCache.h>
@implementation SBMediaOverLayer
{
    CALayer *_highLightLayer;
}

- (instancetype)init{
    if (self = [super init]) {
        _allowHighLight = YES;
        _allowClick = YES;
        //注册复用池
    }
    return self;
}
- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        _allowHighLight = YES;
        _allowClick = YES;
        UILongPressGestureRecognizer *press = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(press:)];
        press.minimumPressDuration = 0.02f;
        [self addGestureRecognizer:press];
    }
    return self;
}
- (void)press:(UILongPressGestureRecognizer *)press{
    switch (press.state) {
        case  UIGestureRecognizerStateBegan:
            if (!_allowClick || !_allowHighLight) {
                return;
            }
            if (!_highLightLayer) {
                _highLightLayer = [self highLightLayer];
            }
            [self.layer addSublayer:_highLightLayer];
            break;
        case  UIGestureRecognizerStateEnded:
            [self removeHighLight];
            CGPoint point = [press locationInView:press.view];
            CGPoint sb_point = [self convertPoint:point toView:self];
            if (sb_point.x>=0 && sb_point.x<=self.bounds.size.width && sb_point.y>=0 && sb_point.y<=self.bounds.size.height)
            {
                //事件处理
            }
            break;
        case  UIGestureRecognizerStateCancelled:
             [self removeHighLight];
            break;
        default:
            break;
    }
}
- (CALayer *)highLightLayer{
    CALayer *layer = [[CALayer alloc] init];
    layer.frame = self.bounds;
    
    layer.backgroundColor = [UIColor colorWithWhite:0.7 alpha:0.5].CGColor;
    if (_highLightColor) {
        layer.backgroundColor = _highLightColor.CGColor;
    }
    return layer;
}


- (void)removeHighLight{
    if (_highLightLayer) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.05f*NSEC_PER_SEC), dispatch_get_main_queue(), ^{
            [_highLightLayer removeFromSuperlayer];
        });
        
    }
}
@end
#define LinkHighLightColor [UIColor colorWithRed:0 green:0 blue:200/255.f alpha:0.5]

@interface SBImageOverBorad()
@property (nonatomic) UIImageView *imageView;
@property (nonatomic) UILabel *titleLabel;
@end
@implementation SBImageOverBorad
- (instancetype)initWithFrame:(CGRect)frame imageUrl:(NSString *)imageUrl title:(NSString *)title{
    
    self = [self initWithFrame:frame];
    [self imageUrl:imageUrl];
    [self title:title];
    return self;
}
- (void)imageUrl:(NSString *)imageUrl{
    if ([imageUrl hasPrefix:@"http"]) {
        //网络图片
        [_imageView sd_setImageWithURL:[NSURL URLWithString:imageUrl]];
    }else{
        //本地图片
        _imageView.image = [UIImage imageNamed:imageUrl];
    }
}
- (void)title:(NSString *)title{
    self.titleLabel.text = title;
}
- (instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame]) {
        _imageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, self.bounds.size.width, self.bounds.size.height-20)];
        _imageView.backgroundColor = [UIColor colorWithWhite:0.8 alpha:1];
        [self addSubview:_imageView];
    }
    return self;
}

- (UILabel *)titleLabel{
    if (_titleLabel == nil) {
        _titleLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, CGRectGetMaxY(_imageView.frame), _imageView.bounds.size.width, 20)];
        _titleLabel.textColor = [UIColor grayColor];
        _titleLabel.font = [UIFont systemFontOfSize:14];
        [self addSubview:_titleLabel];
    }
    return _titleLabel;
}
@end
