//
//  SBTextLayout.m
//  SBTextStoryBorad
//
//  Created by qyb on 2017/7/6.
//  Copyright © 2017年 qyb. All rights reserved.
//

#import "SBTextLayout.h"
#import "SBTextContent.h"
#import "SBMediaOverLayer.h"
@interface SBTextMediaLayout :NSObject
@property (nonatomic) SBTextContentUnit *object;
@property (nonatomic) NSInteger index;
@property (nonatomic) CGRect frame;
@property (nonatomic) CGRect maxFrame;

@end
@implementation SBTextMediaLayout

@end

@interface SBTextLayout()
///< CTFrameSetter
@property (nonatomic,readwrite) CTFramesetterRef frameSetter;
///< CTFrame
@property (nonatomic,readwrite) CTFrameRef frame;

@property (nonatomic,assign,readwrite) CGSize autoSize;
@property (nonatomic) NSArray *linesArray;
@end
@implementation SBTextLayout
{
    NSInteger _index;
    NSMutableArray *_mediaLayouts;
    NSMutableArray <SBTextContentLink *>*_links;
    CGSize _size;
}
- (instancetype)init{
    if (self = [super init]) {
        _mediaLayouts = [NSMutableArray new];
        _links = [NSMutableArray new];
    }
    return self;
}
- (void)drawInContext:(CGContextRef)context
               size:(CGSize)size
               cancel:(BOOL (^)(void))cancel{
    if (cancel()) return;
    CGContextSaveGState(context);
    CGContextSetTextMatrix(context,CGAffineTransformIdentity);
    CGContextTranslateCTM(context,0,size.height);
    CGContextScaleCTM(context,1.0,-1.0);
    _size = size;
    CTFramesetterRef framesetter = CTFramesetterCreateWithAttributedString((CFAttributedStringRef)_attributedText);
    _frameSetter = framesetter;
    CGSize tmpSize = CTFramesetterSuggestFrameSizeWithConstraints(framesetter, CFRangeMake(0,0), NULL, CGSizeMake(size.width-_contentInsets.left*2, MAXFLOAT), NULL);
    _autoSize = tmpSize;
    CGMutablePathRef path = [self calculatePathWithSize:size];
    CTFrameRef frame = CTFramesetterCreateFrame(framesetter, CFRangeMake(0, 0), path, NULL);
    _frame = frame;
    [self mediaLayoutSetting];

    CTFrameDraw(frame, context);
    CGPathRelease(path);
    CGContextRestoreGState(context);
//    CFRelease(framesetter);
//    CFRelease(frame);
}
- (void)removeAllLayer{
    
}
- (NSArray <SBMediaOverLayer*>*)addMediaLayer{
    NSMutableArray *overs = [NSMutableArray new];
    for (SBTextMediaLayout *meidaLayout in _mediaLayouts) {
        SBMediaOverLayer *over = nil;
        if ([meidaLayout.object isKindOfClass:[SBTextContentImage class]]) {
            over = [[SBImageOverBorad alloc] initWithFrame:meidaLayout.frame imageUrl:((SBTextContentImage *)meidaLayout.object).url title:((SBTextContentImage *)meidaLayout.object).title];
        }
        [overs addObject:over];
    }
    return overs;
}
- (void)configLayoutWithContent:(SBTextContent *)content{
    _attributedText = [NSMutableAttributedString new];
    for (SBTextContentUnit *unit in content.unitsArray) {
        //换行
        if ([unit isMemberOfClass:[SBTextContentBrline class]]) {
            [self brline];
        }else if ([unit isMemberOfClass:[SBTextContentParagraph class]]){
            //段落
            [self brparagraph];
        }else if ([unit isMemberOfClass:[SBTextContentText class]]){
            //文本
            [self nomalText:unit];
        }else if ([unit isMemberOfClass:[SBTextContentLink class]]){
            //链接
            [self link:unit];
        }
        else if ([unit isMemberOfClass:[SBTextContentImage class]]){
            //链接
            [self image:unit];
        }
    }
    
}
- (void)brline{
    [_attributedText appendAttributedString:[[NSAttributedString alloc] initWithString:@"\n"]];
}
- (void)image:(SBTextContentUnit *)unit{
    
    [_attributedText appendAttributedString:[self imageAttributeSetting:(SBTextContentImage *)unit]];
    SBTextMediaLayout *mediaLayout = [[SBTextMediaLayout alloc] init];
    mediaLayout.object = unit;
    mediaLayout.index = _mediaLayouts.count;
    [_mediaLayouts addObject:mediaLayout];
}
- (void)brparagraph{
    [self brline];
    [self brline];
}
- (void)nomalText:(SBTextContentUnit *)unit{
    NSMutableDictionary *textAttributeDict = [self unitAttribute:(SBTextContentText *)unit];
    [textAttributeDict setObject:(id)[self defaultsParagraphAttribute] forKey:(id)kCTParagraphStyleAttributeName];
    
    NSAttributedString *textAttribute = [[NSAttributedString alloc] initWithString:((SBTextContentText *)unit).text attributes:textAttributeDict];
    [_attributedText appendAttributedString:textAttribute];
}
- (void)link:(SBTextContentUnit *)unit{
    NSMutableDictionary *textAttributeDict = [NSMutableDictionary new];
    [textAttributeDict setObject:(id)[self defaultsParagraphAttribute] forKey:(id)kCTParagraphStyleAttributeName];
    [self linkAttributeSetting:textAttributeDict contentLink:(SBTextContentLink *)unit];
    NSAttributedString *textAttribute = [[NSAttributedString alloc] initWithString:((SBTextContentLink *)unit).url attributes:textAttributeDict];
    ((SBTextContentLink *)unit).range = NSMakeRange(_attributedText.length, ((SBTextContentLink *)unit).title.length);
    [_links addObject:(SBTextContentLink *)unit];
    [_attributedText appendAttributedString:textAttribute];
    
    
}
- (void)clickLinksAtPoint:(CGPoint)point view:(UIView *)view callBack:(void(^)(NSString *title,NSString *url))callBack{
     CFIndex index = [self touchContentOffsetAtPoint:point view:view];
    if (index == -1) {
        return;
    }
    for (SBTextContentLink *link in _links) {
        if (NSLocationInRange(index,link.range)) {
            callBack(link.title,link.url);
        }
    }
}
#pragma mark - formatsetting
- (void)mediaLayoutSetting{
    CTFrameRef textFrame = _frame;
    NSArray *lines = (NSArray *)CTFrameGetLines(textFrame);
    _linesArray = lines;
    NSUInteger lineCount = [lines count];
    CGPoint lineOrigins[lineCount];
    CTFrameGetLineOrigins(textFrame, CFRangeMake(0, 0), lineOrigins);
    NSInteger imageIndex = 0;
    for (int lIndex = 0; lIndex < lineCount; lIndex++) {
        NSLog(@"orgins:%f",lineOrigins[lIndex].y);
        CTLineRef line = (__bridge CTLineRef)lines[lIndex];
        NSArray * runObjArray = (NSArray *)CTLineGetGlyphRuns(line);
        for (int rIndex = 0; rIndex < runObjArray.count; rIndex ++) {
            CTRunRef run = (__bridge CTRunRef)runObjArray[rIndex];
            NSDictionary *runAttributes = (NSDictionary *)CTRunGetAttributes(run);
            CTRunDelegateRef delegate = (__bridge CTRunDelegateRef)[runAttributes valueForKey:(id)kCTRunDelegateAttributeName];
            if (delegate == nil) {
                continue;
            }
            
            id metaDic = CTRunDelegateGetRefCon(delegate);
            if (![metaDic isKindOfClass:[SBRunDelegateCallBacks class]]) {
                continue;
            }
            
            CGRect runBounds;
            CGFloat ascent;
            CGFloat descent;
            runBounds.size.width = CTRunGetTypographicBounds(run, CFRangeMake(0, 0), &ascent, &descent, NULL);
            runBounds.size.height = ascent + descent;
            
            CGFloat xOffset = CTLineGetOffsetForStringIndex(line, CTRunGetStringRange(run).location, NULL);
//            lIndex = (int)lineCount - lIndex-1;
            runBounds.origin.x = lineOrigins[lIndex].x + xOffset;
            runBounds.origin.y = _size.height-lineOrigins[lIndex].y-runBounds.size.height;
            runBounds.origin.y -= descent;
            
            CGPathRef pathRef = CTFrameGetPath(textFrame);
            CGRect colRect = CGPathGetBoundingBox(pathRef);
            CGRect delegateBounds = CGRectOffset(runBounds, colRect.origin.x, colRect.origin.y);
            delegateBounds.size.height = delegateBounds.size.height<_size.height?delegateBounds.size.height:_size.height;
            delegateBounds.size.width = delegateBounds.size.width<_size.width-delegateBounds.origin.y?delegateBounds.size.width:_size.width;
            if (imageIndex >= _mediaLayouts.count) {
                return;
            }
            SBTextMediaLayout *layout = _mediaLayouts[imageIndex];
            layout.frame = delegateBounds;
            imageIndex ++;
        }
    }
}
- (NSAttributedString *)imageAttributeSetting:(SBTextContentImage *)unit{
    CTRunDelegateCallbacks callbacks;
    memset(&callbacks, 0, sizeof(CTRunDelegateCallbacks));
    callbacks.version = kCTRunDelegateVersion1;
    callbacks.getAscent = ascentCallback;
    callbacks.getDescent = descentCallback;
    callbacks.getWidth = widthCallback;
    CTRunDelegateRef delegate = CTRunDelegateCreate(&callbacks, (__bridge void *)(unit.runDelegate));
    unichar objectReplacementChar = 0xFFFC;
    NSString * content = [NSString stringWithCharacters:&objectReplacementChar length:1];
    NSMutableAttributedString *attribute = [[NSMutableAttributedString alloc] initWithString:content];
    CFAttributedStringSetAttribute((CFMutableAttributedStringRef) attribute, CFRangeMake(0, content.length), kCTRunDelegateAttributeName, delegate);
    return attribute;
}
- (void)linkAttributeSetting:(NSMutableDictionary *)linkAttributeDict contentLink:(SBTextContentLink *)content{
    [linkAttributeDict setObject:content.color?:[UIColor blueColor] forKey:(id)NSForegroundColorAttributeName];
    [linkAttributeDict setObject:content.font?:_font forKey:(id)NSFontAttributeName];
    CGFloat lineSpacing = content.lineSpacing > 0?:_lineSpacing;
    CFNumberRef number = CFNumberCreate(kCFAllocatorDefault,kCFNumberSInt8Type,&lineSpacing);
    [linkAttributeDict setObject:(__bridge id)number forKey:(id)kCTKernAttributeName];
}
- (NSMutableDictionary *)unitAttribute:(SBTextContentText *)content{
    UIFont *font = [content.font copy]?:_font;
//    CTFontRef ctfont = CTFontCreateWithName((CFStringRef)font.fontName,font.pointSize,NULL);
    UIColor *textColor = [content.color copy]?:_textColor;
    CGFloat fontSpacing = _fontSpacing;
    NSMutableDictionary * attributes = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                 font,NSFontAttributeName,
                                textColor,NSForegroundColorAttributeName,@(fontSpacing),NSKernAttributeName,nil];
    return attributes;
}
- (CTParagraphStyleRef)defaultsParagraphAttribute{
    //创建文本对齐方式
    CTTextAlignment alignment = (CTTextAlignment)_textAlignment;//
    CTParagraphStyleSetting alignmentStyle;
    alignmentStyle.spec=kCTParagraphStyleSpecifierAlignment;
    alignmentStyle.valueSize=sizeof(CTTextAlignment);
    alignmentStyle.value=&alignment;
    
    //首行缩进
    CGFloat fristlineindent = 24.0f;
    CTParagraphStyleSetting fristline;
    fristline.spec = kCTParagraphStyleSpecifierFirstLineHeadIndent;
    fristline.value = &fristlineindent;
    fristline.valueSize = sizeof(CGFloat);
    //换行模式
    CTParagraphStyleSetting lineBreakMode;
    CTLineBreakMode lineBreak = (CTLineBreakMode)_lineBreakMode;//kCTLineBreakByWordWrapping;//换行模式
    lineBreakMode.spec = kCTParagraphStyleSpecifierLineBreakMode;
    lineBreakMode.value = &lineBreak;
    lineBreakMode.valueSize = sizeof(CTLineBreakMode);
    
    //行距
    CTParagraphStyleSetting lineSpaceSetting;
    lineSpaceSetting.spec = kCTParagraphStyleSpecifierLineSpacing;
    lineSpaceSetting.value = &_lineSpacing;
    lineSpaceSetting.valueSize = sizeof(CGFloat);
    
    //段前间隔
    CGFloat paragraphspace = 5.0f;
    CTParagraphStyleSetting paragraph;
    paragraph.spec = kCTParagraphStyleSpecifierLineSpacing;
    paragraph.value = &paragraphspace;
    paragraph.valueSize = sizeof(CGFloat);
    
    //组合设置
    CTParagraphStyleSetting settings[] = {
        alignmentStyle,
        fristline,
        lineBreakMode,
        lineSpaceSetting,
        paragraphspace
    };
    CTParagraphStyleRef style = CTParagraphStyleCreate(settings, 5);
   
//    CFRelease(style);
    return style;
}
#pragma mark -  cgpath
- (CGMutablePathRef)calculatePathWithSize:(CGSize)size{
    CGMutablePathRef Path = CGPathCreateMutable();
    CGPathAddRect(Path, NULL ,CGRectMake(_contentInsets.left, _contentInsets.top ,size.width-2*_contentInsets.left , size.height-2*_contentInsets.top));
    return Path;
}
// 将点击的位置转换成字符串的偏移量，如果没有找到，则返回-1
- (CFIndex)touchContentOffsetAtPoint:(CGPoint)point view:(UIView *)view{
    CTFrameRef textFrame = _frame;
    NSArray * lines = _linesArray;
    
    if (!lines) {
        return -1;
    }
    CFIndex count = lines.count;
    
    // 获得每一行的origin坐标
    CGPoint origins[count];
    CTFrameGetLineOrigins(textFrame, CFRangeMake(0,0), origins);
    
    // 翻转坐标系
    CGAffineTransform transform =  CGAffineTransformMakeTranslation(0, view.bounds.size.height);
    transform = CGAffineTransformScale(transform, 1.f, -1.f);
    
    CFIndex idx = -1;
    for (int i = 0; i < count; i++) {
        CGPoint linePoint = origins[i];
        CTLineRef line = CFArrayGetValueAtIndex((CFArrayRef)lines, i);
        // 获得每一行的CGRect信息
        CGRect flippedRect = [self.class getLineBounds:line point:linePoint];
        CGRect rect = CGRectApplyAffineTransform(flippedRect, transform);
        
        if (CGRectContainsPoint(rect, point)) {
            // 将点击的坐标转换成相对于当前行的坐标
            CGPoint relativePoint = CGPointMake(point.x-CGRectGetMinX(rect),
                                                point.y-CGRectGetMinY(rect));
            // 获得当前点击坐标对应的字符串偏移
            idx = CTLineGetStringIndexForPosition(line, relativePoint);
        }
    }
    return idx;
}
+ (CGRect)getLineBounds:(CTLineRef)line point:(CGPoint)point {
    CGFloat ascent = 0.0f;
    CGFloat descent = 0.0f;
    CGFloat leading = 0.0f;
    CGFloat width = (CGFloat)CTLineGetTypographicBounds(line, &ascent, &descent, &leading);
    CGFloat height = ascent + descent;
    return CGRectMake(point.x, point.y - descent, width, height);
}
#pragma mark - ct delegate
static CGFloat ascentCallback(void *ref){
    
    return ((__bridge SBRunDelegateCallBacks *)ref).ascent;
}

static CGFloat descentCallback(void *ref){
    return ((__bridge SBRunDelegateCallBacks *)ref).descen;
}

static CGFloat widthCallback(void* ref){
    return ((__bridge SBRunDelegateCallBacks *)ref).width;
}

@end
